/*
 * ===========================
 * VDK Visual Development Kit
 * xdb subsystem
 * Version 2.4.0
 * Revision 0.1
 * September 2004
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _vdkxtreeview_h
#define _vdkxtreeview_h

#include <vdkxdb/vdkxctrl.h>
#include <vdk/vdktreeview.h>
#include <vdk/value_sem_list.h>
#include <vdk/vdkarray.h>

class VDKXTreeView;

class XTreeViewField
{
 public:
  VDKUString name;
  xbShort id,type,len;
  GType gtype;
  bool editable;
  void (*compute)(GtkTreeIter* iter,
        int column, 
        VDKXTable* table,
        VDKXTreeView*);

  XTreeViewField():name(""),id(-1),type(-1),len(0),
  gtype(0),editable(false),compute(NULL) {}
  XTreeViewField(
          char* name,
          xbShort id = -1,
          xbShort type = -1,
          xbShort len = 0, 
          GType gtype = 0,
          bool editable = false,
          void (*compute)(GtkTreeIter*,
                int,
                VDKXTable*,VDKXTreeView*) = NULL
          ):
    name(name),id(id),type(type),len(len),
    gtype(gtype),editable(editable),compute(compute) {}
  ~XTreeViewField() {}
  bool operator==(XTreeViewField& f)
  { return id == f.id; }
  bool operator<(XTreeViewField& f)
  { return id < f.id; }
};

typedef VDKArray<XTreeViewField> XTreeViewFieldArray;


/*!
  \class VDKXTreeView
  \brief data-aware VDKTreeView widget.

  This control can be used to browse or edit an
  entire VDKXTable.
 */

class VDKXTreeView: 
public VDKTreeView, public VDKXControl
{
protected:
  XTreeViewFieldArray fieldArray;
  VDKTreeViewModel* model;
  void SetData(VDKTreeViewModel *model,
                     GtkTreeIter* iter,
                     int z);
public:

VDKXTreeView (VDKForm *owner,int columns = 1);
virtual ~VDKXTreeView() {}

 /*!
    Assign a table field name to a treeview column
    \param column column index
    \param table table that owns the field
    \param fname field name as in table VDKXRecordTemplate
    descriptor structure (see it into xdb/dbf.h)
    \param editable (defaults to false)
    It can hold:
    \arg all field types
    IMPORTANT: call MakeModel() as you finish to assign
    all fields to tree view and before any other action
  */
bool AssignTableFieldToColumn(
					int column,
					VDKXTable* table,
					char* fname, bool editable = false);
/*!
  Add a column to the view other than a database field
    \param column column index
    \param id, column id, if < 0 function in compute arg
      will be called
    \param gtype column type
    \param title column title (defaults to NULL)
    \param editable (defaults to false)
    \param compute a function called if id < 0

*/
bool AddColumn(
            int column, 
            int id, 
            GType gtype, 
            char* title = NULL, 
            bool editable = false,
            void (*compute)(GtkTreeIter*,
                            int,
                            VDKXTable*,
                            VDKXTreeView*) = NULL);

/*!
    Read fields to control
*/
  virtual xbShort /*VDKXTreeView::*/Read();
/*!
  Complete data model after having assigned fields to
  tree view columns
*/
  bool MakeModel();
};



#endif
