/*
 * ===========================
 * VDK Visual Development Kit
 * xdb subsystem
 * Version 2.4.0
 * Revision 0.1
 * September 2004
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#include <vdkxdb/vdkxdb.h>
#include <vdkxdb/vdkxtreeview.h>
#include <vdk/vdkdate.h>
VDKXTreeView::VDKXTreeView (VDKForm *owner,int columns):
  VDKTreeView(owner), VDKXControl() 
{
  model = NULL;
  fieldArray = XTreeViewFieldArray(columns);
}
 
/* 
*/
bool
VDKXTreeView::MakeModel()
{
  GType* types;
  int real_size;
  int t;
  real_size = fieldArray.size();
  types = new GType[real_size+1];
  for(t = 0; t < real_size; t++)
      types[t] = fieldArray[t].gtype; 
  types[t] = G_TYPE_BOOLEAN;
  this->Model = new VDKTreeViewModel(types,real_size+1);
  delete[] types;
  // makes treeview columns
  VDKTreeViewColumn* column = NULL;
  for(t = 0; t < real_size; t++)
    {
  	column = new VDKTreeViewColumn(
				       this,t,(char*) fieldArray[t].name,
               fieldArray[t].editable,real_size);
    }
    column = new VDKTreeViewColumn(
				       this,t,"editable",true,real_size);

   gtk_tree_view_column_set_visible (
      (*this->Columns())[t]->GtkColumn(),false);
  return true;
}
/*
*/
bool 
VDKXTreeView::AddColumn(
            int column, 
            int id, 
            GType gtype, 
            char* title, 
            bool editable,
            void (*compute)(GtkTreeIter*,
                  int,VDKXTable*,
                  VDKXTreeView*))
{
 fieldArray[column] = XTreeViewField(title,
          id,0,0,gtype,editable,compute);  
 return true;
}
/*
 */
bool 
VDKXTreeView::AssignTableFieldToColumn(int column,VDKXTable* t,
			    char* fname, bool editable)
{
  if(t)
    {
      xbShort id = t->GetFieldNo(fname);
      xbShort type = t->GetFieldType(id);
      xbShort len = t->GetFieldLen(id);
      GType gtype;
      switch(type)
      {
        case XB_LOGICAL_FLD:
          gtype = G_TYPE_BOOLEAN;
          break;
        case XB_CHAR_FLD:
        case XB_DATE_FLD:
          gtype = G_TYPE_STRING;
          break;
        case XB_NUMERIC_FLD:
        case XB_FLOAT_FLD:
          gtype = G_TYPE_DOUBLE;
        default:
          gtype = G_TYPE_STRING;
      }
      fieldArray[column] = 
        XTreeViewField(fname,id,type,len,gtype,editable);
      table = t;
      return true;
    }
  else
    return false;
}

/*
 */
xbShort
VDKXTreeView::Read()
{
  xbShort rc = XB_NOT_OPEN;
  char* current_key = NULL;
  int order;
  xbLong recno = 0;
  int real_size;
  VDKXTableIndex* index = NULL;
  VDKTreeViewModel* model = Model;
  if(! table || 
     ! model || 
     ! GTK_WIDGET_REALIZED(this->WrappedWidget())
      )
    return rc;
  real_size = fieldArray.size();
  // stores table pointer
  if ((order = table->Order) >= 0)
    index = table->Index(order);
  if(index)
    {
      current_key = new char[512];
      index->GetCurrentKey(current_key);
    }
  else
    recno = table->GetCurRecNo();
  // clears the model
  model->Clear();
  // read fields into a tuple
  for(rc =  table->First();
      rc == XB_NO_ERROR;
      rc = table->Next())
    {
      GtkTreeIter iter;
      model->AppendBlank(&iter,NULL);
      // sets dbase data to model
      int z;
      for(z = 0; z < real_size; z++)
        if(fieldArray[z].id >= 0)
          SetData(model,&iter,z);
        else if (fieldArray[z].compute)
          fieldArray[z].compute(&iter,z,table,this);
      // sets editable flag to true
      model->SetData( &iter, z, true,-1);
    }
  if(index)
    index->FindKey(current_key);
  else
    table->GetRecord(recno);
  if(current_key)
    delete[] current_key;
  return XB_NO_ERROR;
}

void
VDKXTreeView::SetData(VDKTreeViewModel *model,
                     GtkTreeIter* iter,
                     int z)
{
  calendardate d;
  xbShort rc;
  char* buffer = NULL;
  char* buffer1 = NULL;
  xbShort id = fieldArray[z].id;
  xbShort type = fieldArray[z].type;
  xbShort len = fieldArray[z].len;
  if(id >= 0)
  {
    buffer = new char[len+1];
    buffer1 = new char[len+1];
    rc = table->GetField(id,buffer);
    if(rc == len)
    {
      switch(type)
	     {
	     case 'D':
	       d = MakeDate(buffer);
	       strcpy(buffer1, (char*) d);
	       break;
	     case 'N':
	     case 'F':
         strcpy(buffer1,table->xbase->LTRIM(buffer));
	       break;
	     case 'L':
	       rc = table->GetLogicalField(id);
	       strcpy(buffer1, rc == 0 ? "0" : "1");
	       break;
	     default:
	       strcpy(buffer1,table->xbase->RTRIM(buffer));
	       break;
	     }
        model->SetCell(iter,z,buffer1);
    }
    delete[] buffer;
    delete[] buffer1;
  }
}
